
;DednDave, updated Dec 12, 2011
;Paul Carter "first.asm"
;Converted to MASM/Win32, Console mode
;
;Convert first.asm from Paul Carter's book to work without the C library.
;Write functions to convert from integer to string and string to integer.

        .586
        .MODEL  Flat, StdCall
        OPTION  CaseMap:None

        INCLUDE    \Masm32\Include\Windows.inc
        INCLUDE    \Masm32\Include\Kernel32.inc
        INCLUDE    HexDump1.inc

        INCLUDELIB \Masm32\Lib\Kernel32.lib

;##################################################################

        .DATA

;  This section is where we place pre-defined data values. These
;items are contained in the actual EXE file, so increase it's size.
;Data in this section may be read from or written to, but if an
;attempt is made to execute code, an exception will occur and the
;program will terminate.

;prompt and message strings

prompt1 db '      Enter a number: ', 0
prompt2 db 'Enter another number: ', 0
outmsg1 db 'You entered ', 0
outmsg2 db ' and ', 0
outmsg3 db 'The sum of these is ', 0

varmsg1 db '"input1"', 0
varmsg2 db '"input2"', 0
addrmsg db ' address: ', 0
valmsg  db ' value: ', 0

szCrLf  db 13, 10, 0                    ;carriage return/line feed

;numeric entry error message strings

ErrMsg1 db 'Value Too Large', 0
ErrMsg2 db 'Non-numeric Character in String', 0
ErrMsg3 db '           Try Again: ', 0

;******************************************************************

        .DATA?

;  This section is where we place data values that have no pre-
;defined value. These items are not contained in the EXE file and
;do not increase it's size. Rather, the EXE header simply holds a
;value that tells the operating system how much space is required.
;When the EXE is loaded, the operating system assigns the memory.
;Therefore, no value other than "?" may be used. In DOS programs,
;this memory often had pre-existing values in it, so no assumptions
;could be made about the initial condition. In Windows programs,
;the operating system continually clears unused memory (when idle)
;and the condition is known to be filled with zeros. Like the .DATA
;section, data in this section may be read from or written to, but
;no code may be executed.

input1  dd ?                            ;first input value
input2  dd ?                            ;second input value
hStdOut dd ?                            ;console output handle
hStdInp dd ?                            ;console input handle
ByteCnt dd ?                            ;read/write byte count

InpBuf  db 1024 dup(?)                  ;user input buffer
OutBuf  db 11 dup(?)                    ;numeric output buffer

;##################################################################

        .CODE

;  This section is where executable code is placed. We are also
;allowed to read values in this section, like data. However, we are
;not allowed to write to this section.

;******************************************************************

_main   PROC

;  This is where execution begins. The END directive at the end of
;the file references the "_main" PROC. It does not have to be the
;first code in the section.
;
;  The Windows ABI specifies that EAX, ECX, and EDX contents may
;be destroyed across system calls and that EBX, EBP, ESI, and EDI
;are preserved, as well as the DF (Direction Flag) should be
;cleared (direction = up). While it is not a requirement that the
;main program follow these rules, we try to write our routines
;using them so that functions may be used in other programs. The
;rules apply primarily to "call-back" situations in GUI mode.

;------------------------------------------

;program initialization

        call    ProgInit

;------------------------------------------

;Paul Carter's code, with a few enhancements in the "dump" section

;get and store the first user-entered number

        call    PrintNewLine            ;display new line
        mov     eax, offset prompt1     ;'      Enter a number: '
        call    PrintString             ;display the string
        call    ReadUnsigned            ;read unsigned integer
        mov     input1, eax             ;store the value as input1

;get and store the second user-entered number

        mov     eax, offset prompt2     ;'Enter another number: '
        call    PrintString             ;display the string
        call    ReadUnsigned            ;read unsigned integer
        mov     input2, eax             ;store the value as input2

;add the two numbers together and store in EBX

        mov     eax, input1             ;EAX = dword at input1
        add     eax, input2             ;EAX += dword at input2
        mov     ebx, eax                ;EBX = EAX, sum is saved in EBX

;hex dump of registers and flags

        call    DumpRegs                ;dump register values in hex
        call    PrintNewLine            ;display new line

;show the address of "input1"

        mov     eax, offset varmsg1     ;'"input1"'
        call    PrintString             ;display the string
        mov     eax, offset addrmsg     ;' address: '
        call    PrintString             ;display the string
        mov     eax, offset input1      ;show the address of "input1"
        call    PrintHexD               ;display the dword as hex
        call    PrintNewLine            ;display new line

;show the address of "input2"

        mov     eax, offset varmsg2     ;'"input2"'
        call    PrintString             ;display the string
        mov     eax, offset addrmsg     ;' address: '
        call    PrintString             ;display the string
        mov     eax, offset input2      ;show the address of "input2"
        call    PrintHexD               ;display the dword as hex
        call    PrintNewLine            ;display new line
        call    PrintNewLine            ;display new line

;show the value of "input1" in hex

        mov     eax, offset varmsg1     ;'"input1"'
        call    PrintString             ;display the string
        mov     eax, offset valmsg      ;' value: '
        call    PrintString             ;display the string
        mov     eax, input1             ;EAX = dword at input1
        call    PrintHexD               ;display the dword as hex
        call    PrintNewLine            ;display new line

;show the value of "input2" in hex

        mov     eax, offset varmsg2     ;'"input2"'
        call    PrintString             ;display the string
        mov     eax, offset valmsg      ;' value: '
        call    PrintString             ;display the string
        mov     eax, input2             ;EAX = dword at input2
        call    PrintHexD               ;display the dword as hex
        call    PrintNewLine            ;display new line

;hex byte dump of input1/input2 data area

        call    PrintNewLine            ;print new-line
        mov     eax, offset input1      ;start at input1
        mov     ecx, 8                  ;dump 8 bytes
        call    DumpBytes               ;dump memory in hex bytes
        call    PrintNewLine            ;display new line

;next print out result message as series of steps

        mov     eax, offset outmsg1     ;'You entered '
        call    PrintString             ;display the string
        mov     eax, input1             ;EAX = dword at input1
        call    PrintUnsigned           ;display the value as unsigned decimal
        mov     eax, offset outmsg2     ;' and '
        call    PrintString             ;display the string
        mov     eax, input2             ;EAX = dword at input2
        call    PrintUnsigned           ;display the value as unsigned decimal
        call    PrintNewLine            ;display new line

        mov     eax, offset outmsg3     ;'The sum of these is '
        call    PrintString             ;display the string
        mov     eax, ebx                ;EAX = saved sum from EBX
        call    PrintUnsigned           ;display the sum as unsigned decimal
        call    PrintNewLine            ;display new line

;------------------------------------------

;program termination

;may be written as: INVOKE ExitProcess, 0

        push    0
        CALL    ExitProcess             ;system:kernel32 call

_main   ENDP

;******************************************************************

Str2Uint PROC

;Convert an ASCII Decimal String to a 32-Bit Unsigned Integer
;
;Call With: EAX = address of zero-terminated ASCII numeric string
;
;  Returns: EAX = status:
;                 0 = no error, EDX is valid
;                 1 = "value too large" error
;                 2 = "non-numeric character in string" error
;           EDX = 32-bit unsigned integer if EAX = 0
;                 undefined if EAX is not 0
;
;Also Uses: ECX, all other registers are preserved

;------------------------------------------------------------------
;
;  Here, we strip off the ASCII bits in each decimal digit, then
;use "Horner's Rule" to convert from base 10 to base 2. We use the
;EDX register as an accumulator to keep the sum. As each new
;decimal digit is encountered, we multiply the accumulated value by
;10, then add the new digit. When we encounter a null (0) characer,
;we have reached the end of the ASCII decimal string.
;
;  What makes this function tricky is the fact that the user may
;input values that are too large or enter ASCII characters that are
;non-numeric. For this reason, we return a "status" condition in
;EAX. This is the standard method of reporting results in Windows
;functions. Notice that these types of problems do not occur when
;converting in the opposite direction (binary-to-decimal).

;------------------------------------------------------------------

        xor     ecx, ecx                ;zero the high part of ECX
        mov     edx, ecx                ;zero the "accumulator"

Str2U0: mov     cl, [eax]               ;get the ASCII character
        jecxz   Str2U2                  ;if it's zero, we're done

        xor     cl, 30h                 ;toggle the ASCII bits
        inc     eax                     ;point to the next character
        cmp     cl, 9                   ;valid numeric character ?
        ja      Str2U3                  ;no - set error status exit

        push    eax                     ;yes - save the string pointer
        mov     eax, 10                 ;multiplier = 10
        mul     edx                     ;EDX:EAX = 10 x accumulator
        or      edx, edx                ;is it too large ?
        mov     edx, eax                ;update the accumulator
        pop     eax                     ;recall the string pointer
        jnz     Str2U1                  ;too large - error status exit

        add     edx, ecx                ;add the new digit to the accum
        jnc     Str2U0                  ;if it carries, value is too large

Str2U1: mov     eax, 1                  ;status = value too large
        ret

Str2U2: xor     eax, eax                ;status = no error
        ret

Str2U3: mov     eax, 2                  ;status = non-numeric character
        ret

Str2Uint ENDP

;******************************************************************

Uint2Str PROC

;Convert a 32-Bit Unsigned Integer to ASCII Decimal String
;
;Call With: EAX = unsigned binary integer
;
;  Returns: EAX = address of zero-terminated ASCII decimal string
;           ECX = length of ASCII decimal string
;
;Also Uses: EDX, all other registers are preserved
;           the "OutBuf" buffer is used to hold the string

;------------------------------------------------------------------

;  A simple method to convert binary values to decimal is to
;repeatedly divide the binary value by 10. For each division step,
;the remainder becomes the new decimal digit and the quotient
;becomes the dividend for the next iteration. Decimal digits are
;created in low-to-high order by this method, so they are placed
;into the string from right-to-left. Notice that the string
;address that is returned by our function for a 6-digit number
;will be different than that of a 5-digit number. When the quotient
;is zero, the binary value has completely been converted to decimal
;digits. ASCII decimal characters are numbered from 48 to 57 (30h
;to 39h). So, to convert a binary value of 0-9 into ASCII
;characters '0'-'9', we add 30h to the binary value. This can be
;done using an OR, XOR, or ADD instruction. This method is
;sometimes called "Divide-And-Conquer", but it is actually a
;reverse implementation of "Horner's Rule".
;
;  On x86 processors, the DIV instruction comes in different forms.
;You can perform byte, word, or dword division, and there is also
;an IDIV instruction that will perform signed division. When DIV is
;used in the dword form, a 64-bit dividend in EDX:EAX is divided by
;a dword divisor, which may be a register or a memory operand. Once
;the DIV instruction has executed, the 32-bit quotient is in EAX
;and the 32-bit remainder (or "modulus") is in EDX. If the
;resulting quotient is too large to fit into the 32 bits of EAX, a
;Divide Overflow error occurs. This can happen if the dividend is
;very large or the divisor is very small. Also, if an attempt is
;made to divide by zero, a Divide By Zero error occurs. Either of
;these errors will generate an exception and, if no exception
;handler has been installed, the program will exit with an error
;message. So, when we use DIV or IDIV, we take special care to
;ensure that these conditions do not occur.
;
;  The DIV and IDIV instructions are not very fast (something like
;80 clock cycles, depending on which CPU is used). There are
;methods to speed up this operation, most notably "Multiply-To-
;Divide", where a number is multiplied by the inverse of the
;divisor, and divided by a power of 2. The x86 MUL instruction
;executes in ~8 clock cycles, so speed can be enhanced
;considerably. The code for this is somewhat involved and it is an
;advanced technique. Another technique that is often employed is
;to use a larger divisor. For example, we could divide by 10,000,
;then convert 4 decimal digits to ASCII in one step.
;
;  However, it is important to understand that optimal speed is not
;always essential. For our purposes, it is unlikely that the user
;would notice a difference between 70 and 700 clock cycles, as the
;function is only call'ed a few times. If we wanted to convert
;hundreds of thousands of values to write to a file, we might try
;to select a more optimal solution.
;
;  Writing binary-to-decimal routines is fun and interesting. For
;that reason, there are a wide variety of code samples available on
;the web. This version is about as simple as it gets, although we
;added a little code to return the string length in ECX.

;------------------------------------------------------------------

        push    edi
        push    ebx
        mov     ecx, 10                 ;ECX = divisor
        mov     edi, offset OutBuf+sizeof OutBuf-1
        xor     ebx, ebx                ;EBX = initial length = 0

Uin2S0: xor     edx, edx                ;zero the high dword
        dec     edi                     ;point to next string byte
        div     ecx                     ;divide EDX:EAX by 10
        or      dl, 30h                 ;set ASCII bits in remainder
        inc     ebx                     ;add 1 to string length
        mov     [edi], dl               ;store the numeric character
        or      eax, eax                ;any quotient left ?
        jnz     Uin2S0                  ;loop until done

        mov     ecx, ebx                ;ECX = string length
        mov     eax, edi                ;EAX = address of first char
        pop     ebx
        pop     edi

        ret

Uint2Str ENDP

;******************************************************************

StrLength PROC

;Get the Length of a Zero-Terminated String
;
;Call With: EAX = address of zero-terminated string
;
;  Returns: EAX = string length, less terminator
;
;Also Uses: ECX, all other registers are preserved

;------------------------------------------------------------------

;  A brief tutorial on string instructions:
;
;LODS - LOaD String - loads the value at [ESI] into AL/AX/EAX
;STOS - STOre String - stores the value in AL/AX/EAX to [EDI]
;MOVS - MOVe String - moves a value from [ESI] to [EDI]
;SCAS - SCAn String - compares values in AL/AX/EAX with the value at [EDI]
;CMPS - CoMPare String - compares the value at [ESI] with the value at [EDI]
;
;Each instruction comes in different sizes.
;For example, MOVS can be used in different forms:
;
;       movsb    ;byte
;       movsw    ;word
;       movsd    ;dword
;
;They can also be used with a label that infers the size.
;The assembler will choose the correct opcode, based on how the label is defined.
;If String1 is defined as a byte array:
;
;       movs    String1
;
;The assembler uses MOVSB.
;
;SCAS, STOS, and LODS use the AL, AX, or EAX register.
;
;The instructions use ESI and EDI as source and destination indexes (addresses):
;
;MOVS and CMPS use both ESI and EDI.
;SCAS and STOS use EDI only.
;LODS uses ESI only.
;
;Any of the string instructions may be combined with a REP prefix to cause repeat operations.
;The number of iterations is controlled by the count in ECX.
;
;The REP mneumonic is used with MOVS and STOS.
;REPZ or REPNZ are used with CMPS and SCAS.
;REP is not typically used with the LODS instruction,
;as it makes no sense to repeatedly load a value into AL, AX, or EAX.
;
;REPZ (or REPE) repeats while the result of the compare or
;scan is zero, or until the count in ECX expires.
;REPNZ (or REPNE) repeats while the result of the compare
;or scan is not zero, or until the count in ECX expires.
;
;For all the string instructions, the addresses in ESI and/or EDI are
;incremented or decremented according to size. For example, if MOVSD
;is used, both ESI and EDI are incremented or decremented by 4 as
;each dword is moved. The direction flag is used to control direction:
;
;CLD - clears the direction flag (up)
;STD - sets the direction flag (down)

;------------------------------------------------------------------

;  Here, we want to scan for a zero byte. We load the ECX register
;with the maximum count, causing the REP operation to repeat until
;it finds a zero byte. When the scan is complete, we subtract the
;remaining count from the maximum count, less one. The "less one"
;is because we do not want to count the zero byte.

        push    edi
        mov     edi, eax                ;EDI = starting address
        mov     ecx, 0FFFFFFFFh
        mov     al, 0                   ;scan for a 0 byte
        repnz   scasb                   ;scan, repeat if not zero
        mov     eax, 0FFFFFFFEh
        sub     eax, ecx                ;calculate string length
        pop     edi

        ret

StrLength ENDP

;******************************************************************

PrintString PROC

;Display a String to Console
;
;Call With: EAX = address of string to display
;
;  Returns: EAX = number of characters displayed
;           the string is displayed to the console
;
;Also Uses: ECX, EDX, all other registers are preserved

;------------------------------------------------------------------

        push    eax
        call    StrLength
        pop     edx

;may be written as: INVOKE WriteFile, hStdOut, edx, eax, offset ByteCnt, NULL

        push    NULL                    ;NULL EQU 0
        push    offset ByteCnt
        push    eax                     ;EAX = string length
        push    edx                     ;EDX = string address
        push    hStdOut
        CALL    WriteFile               ;system:kernel32 call

        mov     eax, ByteCnt

        ret

PrintString ENDP

;******************************************************************

PrintNewLine PROC

;Display a Carriage Return/Line Feed Sequence to Console
;
;Call With: Nothing
;
;  Returns: EAX = number of characters displayed (always 2)
;           carriage return/line feed is displayed to the console
;
;Also Uses: ECX, EDX, all other registers are preserved

;------------------------------------------------------------------

        mov     eax, offset szCrLf
        jmp     PrintString

PrintNewLine ENDP

;******************************************************************

PrintUnsigned PROC

;Display an Unsigned Integer as an ASCII Decimal String to Console
;
;Call With: EAX = unsigned binary integer value
;
;  Returns: EAX = number of characters displayed
;           the value is displayed to console as ASCII decimal
;
;Also Uses: ECX, EDX, all other registers are preserved
;           the "OutBuf" buffer is used to hold the string

;------------------------------------------------------------------

        call    Uint2Str
        jmp     PrintString

PrintUnsigned ENDP

;******************************************************************

ReadString PROC

;Read a User-Input String from Console Keyboard
;
;Call With: EAX = address of input buffer
;           ECX = size of input buffer, in bytes
;
;  Returns: EAX = length of user-input string
;           EDX = address of user-input string (input buffer)
;
;Also Uses: ECX, all other registers are preserved

;------------------------------------------------------------------

        push    edi
        mov     edi,eax

;may be written as: INVOKE ReadFile, hStdInp, eax, ecx, offset ByteCnt, NULL

        push    NULL                    ;NULL EQU 0
        push    offset ByteCnt
        push    ecx                     ;ECX = buffer length
        push    eax                     ;EAX = buffer address
        push    hStdInp
        CALL    ReadFile                ;system:kernel32 call

        mov     ecx, ByteCnt
        mov     edx, edi                ;EDX = address of input buffer
        jecxz   RdStr0

        mov     al, 13                  ;carriage return
        repnz   scasb                   ;scan for carriage return
        jnz     RdStr0

        dec     edi                     ;remove the carriage return

RdStr0: mov     eax, edi
        mov byte ptr [edi], 0           ;terminate the string
        sub     eax, edx                ;EAX = string length
        pop     edi

        ret

ReadString ENDP

;******************************************************************

ReadUnsigned PROC

;Read an Unsigned Integer from Console Keyboard
;
;  If the user enters a value that is too large or any non-numeric
;characters, a re-try prompt is displayed.
;
;Call With: Nothing
;
;  Returns: EAX = EDX = 32-bit unsigned integer value
;
;Also Uses: ECX, all other registers are preserved
;           the "InpBuf" buffer is used to hold the string

;------------------------------------------------------------------

RdUns0: mov     eax, offset InpBuf
        mov     ecx, sizeof InpBuf
        call    ReadString              ;read user string

        mov     eax, offset InpBuf
        call    Str2Uint                ;convert to binary
        or      eax, eax                ;any error ?
        jz      RdUns3                  ;no - exit with value

        cmp     eax, 1                  ;yes - value too large ?
        jz      RdUns1                  ;yes - display "too large"

        mov     eax, offset ErrMsg2     ;'Non-numeric Character in String'
        jmp     RdUns2

RdUns1: mov     eax, offset ErrMsg1     ;'Value Too Large'

RdUns2: call    PrintString
        call    PrintNewLine
        mov     eax, offset ErrMsg3     ;'           Try Again: '
        call    PrintString
        jmp     RdUns0                  ;re-attempt user input

RdUns3: mov     eax, edx                ;return value in EAX
        ret

ReadUnsigned ENDP

;******************************************************************

ProgInit   PROC

;Program Initialization
;
;  Initializes standard output handle (hStdOut), standard input
;handle (hStdInp), and sets console mode for processed input.
;
;Call With: Nothing
;
;  Returns: EAX = nonzero if SetConsoleMode is successful
;           hStdOut = standard output handle
;           hStdInp = standard input handle
;           console mode is set for processed input
;
;Also Uses: ECX, EDX, all other registers are preserved

;------------------------------------------------------------------

;get and store standard output handle

;  We will be using the standard output handle with the WriteFile
;function to display characters and strings in the console window.

;may be written as: INVOKE GetStdHandle, STD_OUTPUT_HANDLE

        push    STD_OUTPUT_HANDLE       ;STD_OUTPUT_HANDLE EQU -11
        CALL    GetStdHandle            ;system:kernel32 call

        mov     hStdOut, eax

;------------------------------------------

;get and store standard input handle

;  We will be using the standard input handle with the ReadFile
;function to acquire user input from the console keyboard. This
;handle is also used in the next section, where we set the console
;input control flags.

;may be written as: INVOKE GetStdHandle, STD_INPUT_HANDLE

        push    STD_INPUT_HANDLE        ;STD_INPUT_HANDLE       EQU -10
        CALL    GetStdHandle            ;system:kernel32 call

        mov     hStdInp, eax

;------------------------------------------

;set console input control flags

;  These flags control the behavior of console keyboard input. They may
;already be set, but we want to ensure that our program works correctly.
;These flags allow us to simplify our code, in that we do not have to
;echo characters to the screen as they are typed or handle backspace
;and other editing keys or Ctrl-C.
;
;ENABLE_PROCESSED_INPUT EQU 1
;CTRL-C is processed by the system and is not placed in the input buffer.
;If the input buffer is being read by ReadFile or ReadConsole, other
;control keys are processed by the system and are not returned in the
;ReadFile or ReadConsole buffer. If the ENABLE_LINE_INPUT mode is also
;enabled, backspace, carriage return, and line feed characters are
;handled by the system.
;
;ENABLE_LINE_INPUT      EQU 2
;The ReadFile or ReadConsole function returns only when a carriage
;return character is read. If this mode is disabled, the functions
;return when one or more characters are available.
;
;ENABLE_ECHO_INPUT      EQU 4
;Characters read by the ReadFile or ReadConsole function are written to
;the active screen buffer as they are read. This mode can be used only
;if the ENABLE_LINE_INPUT mode is also enabled.

;EAX = standard input handle (hStdInp)

;may be written as: INVOKE SetConsoleMode, eax,
;                   ENABLE_ECHO_INPUT or ENABLE_LINE_INPUT or ENABLE_PROCESSED_INPUT

        push    ENABLE_ECHO_INPUT or ENABLE_LINE_INPUT or ENABLE_PROCESSED_INPUT
        push    eax
        CALL    SetConsoleMode          ;system:kernel32 call

        ret

ProgInit   ENDP

;##################################################################

        END     _main

